unit ShellExt;
// The IExtractIcon shell extension relies on the IPersistFile
// interface, and the IContextMenu, IShellPropertySheet, and
// IDragDrop extensions rely on the IShellExtInit interface. This
// unit defines satellite versions of the support interfaces
// IPersistFile and IShellExtInit, along with container object types
// for each. Together, these objects form a middle layer between the
// totally generic satellite and container object types and the
// totally specific final implementations.

interface
USES Windows, Ole2, Classes, SysUtils, ShellApi, ShlObj,
  DelphCom, DLLDebug;

type
  IMyShellExtInit = class(ISatelliteUnknown)
  public
    function Initialize(pidlFolder:PItemIDList; lpdobj: IDataObject;
      hKeyProgID:HKEY):HResult; virtual; stdcall;
  end;

  IMyPersistFile = class(ISatelliteUnknown)
  public
    function GetClassID(var classID: TCLSID): HResult; virtual;
     stdcall;
    function IsDirty: HResult; virtual; stdcall;
    function Load(pszFileName: POleStr; dwMode: Longint): HResult;
      virtual; stdcall;
    function Save(pszFileName: POleStr; fRemember: BOOL): HResult;
      virtual; stdcall;
    function SaveCompleted(pszFileName: POleStr): HResult;
      virtual; stdcall;
    function GetCurFile(var pszFileName: POleStr): HResult;
      virtual; stdcall;
  end;

  ISEIContainer = class(IContainerUnknown)
  protected
    FShellExtInit : IMyShellExtInit; // Satellite interface
  public
    FNumFiles     : Integer;
    FInitFiles    : TStringList;
    FIDPath       : String;
    Constructor Create;
    destructor Destroy; override;
    function QueryInterface(const WantIID: TIID;
      var ReturnedObject): HResult; override;
  end;

  IPFContainer = class(IContainerUnknown)
  protected
    FPersistFile : IMyPersistFile; // Satellite interface
  public
    FPFFilename   : String;
    Constructor Create;
    destructor Destroy; override;
    function QueryInterface(const WantIID: TIID;
      var ReturnedObject): HResult; override;
  end;

implementation

(****** ISEIContainer ******)
constructor ISEIContainer.Create;
begin
  Inherited Create;
  FShellExtInit := IMyShellExtInit.Create(Self);
  FInitFiles    := TStringList.Create;
end;

destructor ISEIContainer.Destroy;
begin
  FInitFiles.Free;
  FShellExtInit.Free;
  Inherited Destroy;
end;

function ISEIContainer.QueryInterface(const WantIID: TIID;
  var ReturnedObject): HResult;
VAR P : IUnknown;
begin
  Loggit(Format('QueryInterface: %s requested',
    [NameOfIID(WantIID)]));
  Result := Inherited QueryInterface(WantIID, ReturnedObject);
  IF Result = S_OK THEN Exit
  ELSE IF IsEqualIID(WantIID, IID_IShellextInit) THEN
    P := FShellExtInit
  ELSE P := nil;
  Pointer(ReturnedObject) := P;
  IF P = NIL THEN Result := E_NOINTERFACE
  ELSE
    begin
      P.AddRef;
      Result := S_OK;
    end;
end;

(****** IPFContainer ******)
constructor IPFContainer.Create;
begin
  Inherited Create;
  FPFFileName   := '';
  FPersistFile := IMyPersistFile.Create(Self);
end;

destructor IPFContainer.Destroy;
begin
  FPersistFile.Free;
  Inherited Destroy;
end;

function IPFContainer.QueryInterface(const WantIID: TIID;
  var ReturnedObject): HResult;
VAR P : IUnknown;
begin
  Loggit(Format('QueryInterface: %s requested',
    [NameOfIID(WantIID)]));
  Result := Inherited QueryInterface(WantIID, ReturnedObject);
  IF Result = S_OK THEN Exit
  ELSE IF IsEqualIID(WantIID, IID_IPersistFile) THEN
    P := FPersistFile
  ELSE P := nil;
  Pointer(ReturnedObject) := P;
  IF P = NIL THEN Result := E_NOINTERFACE
  ELSE
    begin
      P.AddRef;
      Result := S_OK;
    end;
end;

(****** IMyShellExtInit ******)
function IMyShellExtInit.Initialize(pidlFolder:PItemIDList;
  lpdobj: IDataObject; hKeyProgID:HKEY):HResult;
VAR
  buffer  : ARRAY[0..MAX_PATH] OF Char;
  Medium  : TStgMedium;
  fe      : TFormatEtc;
  N       : Integer;
begin
  IF lpdobj = NIL THEN
    begin
      result := E_FAIL;
      Exit;
    end;
  WITH fe DO
    begin
      cfFormat := CF_HDROP;
      ptd      := NIL;
      dwAspect := DVASPECT_CONTENT;
      lindex   := -1;
      tymed    := TYMED_HGLOBAL;
    end;
  Result := lpdobj.GetData(fe, medium);
  IF failed(Result) THEN Exit;
  SHGetPathFromIDList(pidlFolder, Buffer);
  WITH ISEIContainer(fContainer) DO
    begin
      FIdPath := StrPas(Buffer);
      FNumFiles :=
        DragQueryFile(medium.hGlobal, -1, NIL, 0);
      FInitFiles.Clear;
      IF FNumFiles > 0 THEN
        begin
          FOR N := 0 TO FNumFiles-1 DO
            begin
              DragQueryFile(medium.hGlobal, N, Buffer, 260);
              FInitFiles.Add(StrPas(Buffer));
            end;
          Result := NOERROR;
        end
      ELSE Result := E_FAIL;
    end;
  ReleaseStgMedium(medium);
end;

(****** IMyPersistFile ******)
function IMyPersistFile.GetClassID(var classID: TCLSID): HResult;
begin
  Result := E_FAIL;
end;

function IMyPersistFile.IsDirty: HResult;
begin
  Result := E_FAIL;
end;

function IMyPersistFile.Load(pszFileName: POleStr; dwMode: Longint): HResult;
begin
  IPFContainer(FContainer).FPFFilename :=
    WideCharToString(pszFilename);
  Result := S_OK;
end;

function IMyPersistFile.Save(pszFileName: POleStr; fRemember: BOOL): HResult;
begin
  Result := E_FAIL;
end;

function IMyPersistFile.SaveCompleted(pszFileName: POleStr): HResult;
begin
  Result := E_FAIL;
end;

function IMyPersistFile.GetCurFile(var pszFileName: POleStr): HResult;
begin
  Result := E_FAIL;
end;

end.
