unit DelphCom;
// "Generic" objects for creating COM objects in Delphi.
// ISatelliteUnknown is an interface object that's meant
// to be contained by IContainerUnknown. Each actual COM
// object that needs to support more than one interface
// will descend from IContainerUnknown, and will override
// QueryInterface.
interface

uses
  Windows, Ole2;

type
  IContainerUnknown = class;

  ISatelliteUnknown = class(IUnknown)
  // Meant to be contained by an IContainerUnknown
  // Reflects the three IUnknown functions to its container.
  protected
    FContainer: IContainerUnknown;
  public
    constructor Create(vContainer: IContainerUnknown);
    function QueryInterface(const WantIID: TIID;
      var ReturnedObject): HResult; override;
    function AddRef: Longint; override;
    function Release: Longint; override;
  end;

  IContainerUnknown = class(IUnknown)
  protected
    FRefCount: Integer;
  public
    constructor Create;
    destructor Destroy; override;
    {IUnknown functions}
    function QueryInterface(const WantIID: TIID;
      var ReturnedObject): HResult; override;
    function AddRef: LongInt; override;
    function Release: LongInt; override;
  end;

  IMyClassFactory = class(IClassFactory)
  private
    FRefCount : Integer;
  public
    constructor Create;
    destructor Destroy; override;
    function QueryInterface(const WantIID: TIID;
      var ReturnedObject): HResult; override;
    function AddRef: LongInt; override;
    function Release: LongInt; override;
    // descendant MUST implement CreateInstance
    function LockServer(fLock: BOOL): HResult; override;
  end;

function DLLCanUnloadNow: HResult; stdcall; export;

implementation

var
  DllRefCount: Integer;

(****** ISatelliteUnknown ******)
constructor ISatelliteUnknown.Create(vContainer: IContainerUnknown);
begin
  FContainer := vContainer;
end;

function ISatelliteUnknown.QueryInterface(const WantIID: TIID;
  var ReturnedObject): HResult;
begin
  Result := FContainer.QueryInterface(WantIid, ReturnedObject);
end;

function ISatelliteUnknown.AddRef: LongInt;
begin
  Result := FContainer.AddRef;
end;

function ISatelliteUnknown.Release: LongInt;
begin
  Result := FContainer.Release;
end;

(****** IContainerUnknown ******)
constructor IContainerUnknown.Create;
begin
  inherited Create;
  FRefCount := 0;
  Inc(DllRefCount);
end;

destructor IContainerUnknown.Destroy;
begin
  Dec(DllRefCount);
  inherited Destroy;
end;

function IContainerUnknown.QueryInterface(const WantIID: TIID;
  var ReturnedObject): HResult;
var
  P: IUnknown;
begin
  if IsEqualIID(WantIID, IID_IUnknown) then
    P := Self
  else
    P := nil;
  Pointer(ReturnedObject) := P;
  if P = nil then
    Result := E_NOINTERFACE
  else
  begin
    P.AddRef;
    Result := S_OK;
  end;
end;

function IContainerUnknown.AddRef: LongInt;
begin
  Inc(FRefCount);
  Result := FRefCount;
end;

function IContainerUnknown.Release: LongInt;
begin
  Dec(FRefCount);
  Result := FRefCount;
  if FRefCount = 0 then Free;
end;

(****** IMyClassFactory ******)
constructor IMyClassFactory.Create;
begin
  inherited Create;
  Inc(DllRefCount);
  FRefCount := 0;
end;

destructor IMyClassFactory.Destroy;
begin
  Dec(DllRefCount);
  inherited Destroy;
end;

function IMyClassFactory.QueryInterface(const WantIID: TIID;
  var ReturnedObject): HResult;
begin
  if IsEqualIID(WantIid, IID_IUnknown) or
     IsEqualIID(WantIid, IID_IClassFactory) then
  begin
    Pointer(ReturnedObject) := Self;
    AddRef;
    Result := S_OK;
  end
  else
  begin
    Pointer(ReturnedObject) := nil;
    Result := E_NOINTERFACE;
  end;
end;

function IMyClassFactory.AddRef: LongInt;
begin
  Inc(FRefCount);
  Result := FRefCount;
end;

function IMyClassFactory.Release: LongInt;
begin
  Dec(FRefCount);
  Result := FRefCount;
  if FRefCount = 0 then Free;
end;

function IMyClassFactory.LockServer(fLock: Bool): HResult;
begin
  Result := E_NOTIMPL;
end;

(****** exported function ******)
function DLLCanUnloadNow: hResult; stdcall; export;
begin
  if DllRefCount = 0 then
    Result := S_OK
  else
    Result := S_FALSE;
end;

initialization
  DllRefCount := 0;
end.

